@@ -1,19 +1,33 @@
 /*
-    SPDX-FileCopyrightText: 2008 Omat Holding B .V. <info@omat.nl>
-    SPDX-FileCopyrightText: 2010 Klarälvdalens Datakonsult AB , a KDAB Group company <info@kdab.com>
-    SPDX-FileContributor: Kevin Ottens <kevin@kdab.com>
-    SPDX-FileCopyrightText: 2017 Sandro Kanuß <knauss@kde.org>
+   Copyright (C) 2008 Omat Holding B.V. <info@omat.nl>
 
-    SPDX-License-Identifier: GPL-2.0-or-later
+   Copyright (C) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
+   Author: Kevin Ottens <kevin@kdab.com>
+
+   Copyright (C) 2017 Sandro Kanuß <knauss@kde.org>
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef FAKESERVER_H
 #define FAKESERVER_H
 
-#include <QMutex>
-#include <QTcpServer>
 #include <QTcpSocket>
+#include <QTcpServer>
 #include <QThread>
+#include <QMutex>
 
 Q_DECLARE_METATYPE(QList<QByteArray>)
 
@@ -37,7 +51,7 @@ Q_DECLARE_METATYPE(QList<QByteArray>)
  * start to indicate whether it is message that will be sent by the
  * client ("C: ") or a response that should be sent by the server
  * ("S: "). Or ("D: ") for the exchanged data. Content-length header is added
- * automatically with the current length and also the empty line between Header
+ * automatically with the corrent length and also the empty line between Header
  * and Content. For example:
  * @code
  * C: GET /item HTTP/1.1
@@ -50,7 +64,7 @@ Q_DECLARE_METATYPE(QList<QByteArray>)
  * A line starting with X indicates that the connection should be
  * closed by the server.  This should be the last line in the
  * scenario.
-
+ 
  * A typical usage is something like
  * @code
  * QList<QByteArray> scenario;
@@ -76,16 +90,13 @@ Q_DECLARE_METATYPE(QList<QByteArray>)
  * @endcode
  */
 
-class FakeServer : public QObject
+class FakeServer : public QThread
 {
     Q_OBJECT
 
 public:
-    /**
-     * Each unittest should use a different port so that they can be run in parallel
-     */
-    FakeServer(int port = 5989, QObject *parent = nullptr);
-    ~FakeServer() override;
+    FakeServer(QObject *parent = nullptr);
+    ~FakeServer();
 
     /**
      * Starts the server and waits for it to be ready
@@ -94,6 +105,15 @@ public:
      */
     void startAndWait();
 
+    /**
+     * Starts the fake server
+     *
+     * You should not call this directly.  Use start() instead.
+     *
+     * @reimp
+     */
+    void run() Q_DECL_OVERRIDE;
+
     /**
      * Removes any previously-added scenarios, and adds a new one
      *
@@ -159,18 +179,16 @@ public:
 private Q_SLOTS:
     void newConnection();
     void dataAvailable();
-    void init();
-    void cleanup();
+    void started();
 
 private:
-    void writeServerPart(QTcpSocket *clientSocket, int scenarioNumber);
-    void readClientPart(QTcpSocket *socket, int *scenarioNumber);
+    void writeServerPart(int scenarioNumber);
+    void readClientPart(int scenarioNumber);
 
-    QList<QList<QByteArray>> m_scenarios;
-    QTcpServer *m_tcpServer = nullptr;
+    QList< QList<QByteArray> > m_scenarios;
+    QTcpServer *m_tcpServer;
     mutable QMutex m_mutex;
     QList<QTcpSocket *> m_clientSockets;
-    QThread *m_thread;
     int m_port;
 };
 
