@@ -1,98 +1,83 @@
 /*
-    SPDX-FileCopyrightText: 2010 Tobias Koenig <tokoe@kde.org>
+    Copyright (c) 2010 Tobias Koenig <tokoe@kde.org>
 
-    SPDX-License-Identifier: LGPL-2.0-or-later
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #include "davcollectionsfetchjob.h"
-#include "davjobbase_p.h"
 
-#include "daverror.h"
-#include "davmanager_p.h"
+#include "davmanager.h"
 #include "davprincipalhomesetsfetchjob.h"
-#include "davprotocolbase_p.h"
-#include "utils_p.h"
+#include "davprotocolbase.h"
+#include "utils.h"
+#include "daverror.h"
 
 #include "libkdav_debug.h"
 #include <KIO/DavJob>
 #include <KIO/Job>
 
-#include <QBuffer>
 #include <QColor>
+#include <QtCore/QBuffer>
+#include <QtXmlPatterns/QXmlQuery>
 
 using namespace KDAV;
 
-namespace KDAV
-{
-class DavCollectionsFetchJobPrivate : public DavJobBasePrivate
-{
-public:
-    void principalFetchFinished(KJob *job);
-    void collectionsFetchFinished(KJob *job);
-    void doCollectionsFetch(const QUrl &url);
-    void subjobFinished();
-
-    DavUrl mUrl;
-    DavCollection::List mCollections;
-    uint mSubJobCount = 0;
-
-    Q_DECLARE_PUBLIC(DavCollectionsFetchJob)
-};
-}
-
 DavCollectionsFetchJob::DavCollectionsFetchJob(const DavUrl &url, QObject *parent)
-    : DavJobBase(new DavCollectionsFetchJobPrivate, parent)
+    : DavJobBase(parent), mUrl(url), mSubJobCount(0)
 {
-    Q_D(DavCollectionsFetchJob);
-    d->mUrl = url;
 }
 
 void DavCollectionsFetchJob::start()
 {
-    Q_D(DavCollectionsFetchJob);
-    if (DavManager::davProtocol(d->mUrl.protocol())->supportsPrincipals()) {
-        DavPrincipalHomeSetsFetchJob *job = new DavPrincipalHomeSetsFetchJob(d->mUrl);
-        connect(job, &DavPrincipalHomeSetsFetchJob::result, this, [d](KJob *job) {
-            d->principalFetchFinished(job);
-        });
+    if (DavManager::self()->davProtocol(mUrl.protocol())->supportsPrincipals()) {
+        DavPrincipalHomeSetsFetchJob *job = new DavPrincipalHomeSetsFetchJob(mUrl);
+        connect(job, &DavPrincipalHomeSetsFetchJob::result, this, &DavCollectionsFetchJob::principalFetchFinished);
         job->start();
     } else {
-        d->doCollectionsFetch(d->mUrl.url());
+        doCollectionsFetch(mUrl.url());
     }
 }
 
 DavCollection::List DavCollectionsFetchJob::collections() const
 {
-    Q_D(const DavCollectionsFetchJob);
-    return d->mCollections;
+    return mCollections;
 }
 
 DavUrl DavCollectionsFetchJob::davUrl() const
 {
-    Q_D(const DavCollectionsFetchJob);
-    return d->mUrl;
+    return mUrl;
 }
 
-void DavCollectionsFetchJobPrivate::doCollectionsFetch(const QUrl &url)
+void DavCollectionsFetchJob::doCollectionsFetch(const QUrl &url)
 {
     ++mSubJobCount;
 
-    const QDomDocument collectionQuery = DavManager::davProtocol(mUrl.protocol())->collectionsQuery()->buildQuery();
+    const QDomDocument collectionQuery = DavManager::self()->davProtocol(mUrl.protocol())->collectionsQuery()->buildQuery();
 
-    KIO::DavJob *job = DavManager::self()->createPropFindJob(url, collectionQuery.toString());
-    QObject::connect(job, &KIO::DavJob::result, q_ptr, [this](KJob *job) {
-        collectionsFetchFinished(job);
-    });
+    KIO::DavJob *job = DavManager::self()->createPropFindJob(url, collectionQuery);
+    connect(job, &KIO::DavJob::result, this, &DavCollectionsFetchJob::collectionsFetchFinished);
     job->addMetaData(QStringLiteral("PropagateHttpHeader"), QStringLiteral("true"));
 }
 
-void DavCollectionsFetchJobPrivate::principalFetchFinished(KJob *job)
+void DavCollectionsFetchJob::principalFetchFinished(KJob *job)
 {
     const DavPrincipalHomeSetsFetchJob *davJob = qobject_cast<DavPrincipalHomeSetsFetchJob *>(job);
 
     if (davJob->error()) {
-        if (davJob->canRetryLater()) {
-            // If we have a non-persistent HTTP error code then this may mean that
+        if (davJob->latestResponseCode()) {
+            // If we have a HTTP response code then this may mean that
             // the URL was not a principal URL. Retry as if it were a calendar URL.
             qCDebug(KDAV_LOG) << job->errorText();
             doCollectionsFetch(mUrl.url());
@@ -107,7 +92,7 @@ void DavCollectionsFetchJobPrivate::principalFetchFinished(KJob *job)
     }
 
     const QStringList homeSets = davJob->homeSets();
-    qCDebug(KDAV_LOG) << "Found" << homeSets.size() << "homesets";
+    qCDebug(KDAV_LOG) << "Found " << homeSets.size() << " homesets";
     qCDebug(KDAV_LOG) << homeSets;
 
     if (homeSets.isEmpty()) {
@@ -116,7 +101,7 @@ void DavCollectionsFetchJobPrivate::principalFetchFinished(KJob *job)
         return;
     }
 
-    for (const QString &homeSet : homeSets) {
+    foreach (const QString &homeSet, homeSets) {
         QUrl url = mUrl.url();
 
         if (homeSet.startsWith(QLatin1Char('/'))) {
@@ -134,12 +119,12 @@ void DavCollectionsFetchJobPrivate::principalFetchFinished(KJob *job)
     }
 }
 
-void DavCollectionsFetchJobPrivate::collectionsFetchFinished(KJob *job)
+void DavCollectionsFetchJob::collectionsFetchFinished(KJob *job)
 {
-    Q_Q(DavCollectionsFetchJob);
     KIO::DavJob *davJob = qobject_cast<KIO::DavJob *>(job);
-    const QString responseCodeStr = davJob->queryMetaData(QStringLiteral("responsecode"));
-    const int responseCode = responseCodeStr.isEmpty() ? 0 : responseCodeStr.toInt();
+    const int responseCode = davJob->queryMetaData(QStringLiteral("responsecode")).isEmpty() ?
+                             0 :
+                             davJob->queryMetaData(QStringLiteral("responsecode")).toInt();
 
     // KIO::DavJob does not set error() even if the HTTP status code is a 4xx or a 5xx
     if (davJob->error() || (responseCode >= 400 && responseCode < 600)) {
@@ -164,26 +149,48 @@ void DavCollectionsFetchJobPrivate::collectionsFetchFinished(KJob *job)
         const QString jobUrl = _jobUrl.toDisplayString();
 
         // Validate that we got a valid PROPFIND response
-        QDomDocument response;
-        response.setContent(davJob->responseData(), QDomDocument::ParseOption::UseNamespaceProcessing);
-        QDomElement rootElement = response.documentElement();
-        if (rootElement.tagName().compare(QLatin1String("multistatus"), Qt::CaseInsensitive) != 0) {
+        QDomElement rootElement = davJob->response().documentElement();
+        if (rootElement.tagName().compare(QStringLiteral("multistatus"), Qt::CaseInsensitive) != 0) {
             setError(ERR_COLLECTIONFETCH);
             setErrorTextFromDavError();
             subjobFinished();
             return;
         }
 
-        QByteArray resp = davJob->responseData();
+        QByteArray resp(davJob->response().toByteArray());
+        QBuffer buffer(&resp);
+        buffer.open(QIODevice::ReadOnly);
+
+        QXmlQuery xquery;
+        if (!xquery.setFocus(&buffer)) {
+            setError(ERR_COLLECTIONFETCH_XQUERY_SETFOCUS);
+            setErrorTextFromDavError();
+            subjobFinished();
+            return;
+        }
+
+        xquery.setQuery(DavManager::self()->davProtocol(mUrl.protocol())->collectionsXQuery());
+        if (!xquery.isValid()) {
+            setError(ERR_COLLECTIONFETCH_XQUERY_INVALID);
+            setErrorTextFromDavError();
+            subjobFinished();
+            return;
+        }
+
+        QString responsesStr;
+        xquery.evaluateTo(&responsesStr);
+        responsesStr.prepend(QStringLiteral("<responses>"));
+        responsesStr.append(QStringLiteral("</responses>"));
+
         QDomDocument document;
-        if (!document.setContent(resp, QDomDocument::ParseOption::UseNamespaceProcessing)) {
+        if (!document.setContent(responsesStr, true)) {
             setError(ERR_COLLECTIONFETCH);
             setErrorTextFromDavError();
             subjobFinished();
             return;
         }
 
-        if (!q->error()) {
+        if (!error()) {
             /*
              * Extract information from a document like the following:
              *
@@ -222,6 +229,7 @@ void DavCollectionsFetchJobPrivate::collectionsFetchFinished(KJob *job)
 
             QDomElement responseElement = Utils::firstChildElementNS(responsesElement, QStringLiteral("DAV:"), QStringLiteral("response"));
             while (!responseElement.isNull()) {
+
                 QDomElement propstatElement;
 
                 // check for the valid propstat, without giving up on first error
@@ -230,7 +238,7 @@ void DavCollectionsFetchJobPrivate::collectionsFetchFinished(KJob *job)
                     for (int i = 0; i < propstats.length(); ++i) {
                         const QDomElement propstatCandidate = propstats.item(i).toElement();
                         const QDomElement statusElement = Utils::firstChildElementNS(propstatCandidate, QStringLiteral("DAV:"), QStringLiteral("status"));
-                        if (statusElement.text().contains(QLatin1String("200"))) {
+                        if (statusElement.text().contains(QStringLiteral("200"))) {
                             propstatElement = propstatCandidate;
                         }
                     }
@@ -265,7 +273,7 @@ void DavCollectionsFetchJobPrivate::collectionsFetchFinished(KJob *job)
 
                 // don't add this resource if it has already been detected
                 bool alreadySeen = false;
-                for (const DavCollection &seen : std::as_const(mCollections)) {
+                foreach (const DavCollection &seen, mCollections) {
                     if (seen.url().toDisplayString() == url.toDisplayString()) {
                         alreadySeen = true;
                     }
@@ -277,43 +285,28 @@ void DavCollectionsFetchJobPrivate::collectionsFetchFinished(KJob *job)
 
                 // extract display name
                 const QDomElement propElement = Utils::firstChildElementNS(propstatElement, QStringLiteral("DAV:"), QStringLiteral("prop"));
-                if (!DavManager::davProtocol(mUrl.protocol())->containsCollection(propElement)) {
-                    responseElement = Utils::nextSiblingElementNS(responseElement, QStringLiteral("DAV:"), QStringLiteral("response"));
-                    continue;
-                }
                 const QDomElement displaynameElement = Utils::firstChildElementNS(propElement, QStringLiteral("DAV:"), QStringLiteral("displayname"));
                 const QString displayName = displaynameElement.text();
 
                 // Extract CTag
-                const QDomElement CTagElement = Utils::firstChildElementNS(propElement, //
-                                                                           QStringLiteral("http://calendarserver.org/ns/"),
-                                                                           QStringLiteral("getctag"));
+                const QDomElement CTagElement = Utils::firstChildElementNS(propElement, QStringLiteral("http://calendarserver.org/ns/"), QStringLiteral("getctag"));
                 QString CTag;
                 if (!CTagElement.isNull()) {
                     CTag = CTagElement.text();
                 }
 
                 // extract calendar color if provided
-                const QDomElement colorElement = Utils::firstChildElementNS(propElement, //
-                                                                            QStringLiteral("http://apple.com/ns/ical/"),
-                                                                            QStringLiteral("calendar-color"));
+                const QDomElement colorElement = Utils::firstChildElementNS(propElement, QStringLiteral("http://apple.com/ns/ical/"), QStringLiteral("calendar-color"));
                 QColor color;
                 if (!colorElement.isNull()) {
                     QString colorValue = colorElement.text();
-                    if (QColor::isValidColorName(colorValue)) {
-                        // Color is either #RRGGBBAA or #RRGGBB but QColor expects #AARRGGBB
-                        // so we put the AA in front if the string's length is 9.
-                        if (colorValue.size() == 9) {
-                            QString fixedColorValue = QStringLiteral("#") + colorValue.mid(7, 2) + colorValue.mid(1, 6);
-                            color = QColor::fromString(fixedColorValue);
-                        } else {
-                            color = QColor::fromString(colorValue);
-                        }
+                    if (QColor::isValidColor(colorValue)) {
+                        color.setNamedColor(colorValue);
                     }
                 }
 
                 // extract allowed content types
-                const DavCollection::ContentTypes contentTypes = DavManager::davProtocol(mUrl.protocol())->collectionContentTypes(propstatElement);
+                const DavCollection::ContentTypes contentTypes = DavManager::self()->davProtocol(mUrl.protocol())->collectionContentTypes(propstatElement);
 
                 auto _url = url;
                 _url.setUserInfo(mUrl.url().userInfo());
@@ -325,9 +318,7 @@ void DavCollectionsFetchJobPrivate::collectionsFetchFinished(KJob *job)
                 }
 
                 // extract privileges
-                const QDomElement currentPrivsElement = Utils::firstChildElementNS(propElement, //
-                                                                                   QStringLiteral("DAV:"),
-                                                                                   QStringLiteral("current-user-privilege-set"));
+                const QDomElement currentPrivsElement = Utils::firstChildElementNS(propElement, QStringLiteral("DAV:"), QStringLiteral("current-user-privilege-set"));
                 if (currentPrivsElement.isNull()) {
                     // Assume that we have all privileges
                     collection.setPrivileges(KDAV::All);
@@ -338,7 +329,7 @@ void DavCollectionsFetchJobPrivate::collectionsFetchFinished(KJob *job)
 
                 qCDebug(KDAV_LOG) << url.toDisplayString() << "PRIVS: " << collection.privileges();
                 mCollections << collection;
-                Q_EMIT q->collectionDiscovered(mUrl.protocol(), url.toDisplayString(), jobUrl);
+                Q_EMIT collectionDiscovered(mUrl.protocol(), url.toDisplayString(), jobUrl);
 
                 responseElement = Utils::nextSiblingElementNS(responseElement, QStringLiteral("DAV:"), QStringLiteral("response"));
             }
@@ -348,11 +339,10 @@ void DavCollectionsFetchJobPrivate::collectionsFetchFinished(KJob *job)
     subjobFinished();
 }
 
-void DavCollectionsFetchJobPrivate::subjobFinished()
+void DavCollectionsFetchJob::subjobFinished()
 {
     if (--mSubJobCount == 0) {
         emitResult();
     }
 }
 
-#include "moc_davcollectionsfetchjob.cpp"
